(in-package :it.bese.ucw-user)

(defvar *reddit-application*
  (make-instance 'cookie-session-application
		 :url-prefix "/reddit/"))

(register-application *default-server* *reddit-application*)

(defentry-point "reddit.ucw" (:application *reddit-application*) ()
    (call 'reddit))

(defvar *all-links* '())
(defvar *id-counter* 0)

(defclass reddit-link ()
  ((href :accessor href :initarg :href :initform "http://")
   (title :accessor title :initarg :title :initform "Title")
   (date :accessor date :initform (get-universal-time))
   (points :accessor points :initform 0)
   (id :accessor id :initform (1+ *id-counter*))))

(defconstant +month-names+
  (list "January"
	"February"
	"March"
	"April"
	"May"
	"June"
	"July"
	"August"
	"September"
	"October"
	"November"
	"December"))

(defconstant +day-names+
  (list "Monday"
	"Tuesday"
	"Wednesday"
	"Thursday"
	"Friday"
	"Saturday"
	"Sunday"))

(defmethod render-date ((link reddit-link))
  (multiple-value-bind
	(second minute hour daate month year day-of-week dst-p tz)
      (decode-universal-time (slot-value link 'date))
    (format nil "~0d ~0d ~d ~a / ~2,'0d:~2,'0d:~2,'0d GMT~@d~:[~; (DST)~]"
	    daate
	    (nth (1- month) +month-names+)
	    year
	    (nth day-of-week +day-names+)
	    hour
	    minute
	    second
	    tz
	    dst-p)))


(defcomponent reddit (simple-window-component)
  ((popular-list :accessor popular-list :component (popular-links-viewer))
   (latest-list :accessor latest-list :component (latest-links-viewer)))
  (:default-initargs :title "Reddit.ucw"))

(defmethod render-on ((res response) (red reddit))
  (<:h1 "Reddit.ucw")
  (<:h3 "Just Another Reddit clone")
  (<ucw:a :action (submit-link red) "Submit Link")
  (render-on res (latest-list red))
  (render-on res (popular-list red)))

(defun add-link (&key href title)
  (push (make-instance 'reddit-link :href href :title title) *all-links*))

(defaction submit-link ((page reddit))
  (call 'submit-link-page))

(defcomponent link-viewer (widget-component)
  ()
  (:default-initargs :css-class "linklist"))

(defmethod render-on ((res response) (viewer link-viewer))
  (<:ol
   (loop for item in (sort-items viewer *all-links*)
	 do (render-link viewer item))))

(defaction rate-up ((viewer link-viewer) (link reddit-link))
  (incf (points link)))

(defaction rate-down ((viewer link-viewer) (link reddit-link))
  (decf (points link)))

(defmethod render-link ((viewer link-viewer) (link reddit-link))
  (<:li (<:a :href (href link) (<:as-html (title link))) " "
	(<ucw:a :action (rate-up viewer link) (<:as-html "Up")) " "
	(<ucw:a :action (rate-down viewer link) (<:as-html "Down"))
	(<:br)
	(<:as-html (render-date link) " Points: " (points link))))

(defun sort-list (list &key key)
  (sort (copy-list list) #'> :key key))

(defcomponent latest-links-viewer (link-viewer)
  ())

(defmethod sort-items ((viewer latest-links-viewer) items)
  (sort-list items :key #'date))

(defmethod render-on ((res response) (viewer latest-links-viewer))
  (<:h2 (<:as-is "Latest Links"))
  (call-next-method))

(defcomponent popular-links-viewer (link-viewer)
  ())

(defmethod sort-items ((viewer popular-links-viewer) items)
  (sort-list items :key #'points))

(defmethod render-on ((res response) (viewer popular-links-viewer))
  (<:h2 (<:as-is "Most Popular"))
  (call-next-method))

(defcomponent submit-link-page (simple-window-component)
  ((href :accessor href :initarg :href :initform "http://")
   (link-title :accessor link-title :initarg :link-title :initform "Title"))
  (:default-initargs :title "Submit Link"))

(defmethod render-on ((res response) (page submit-link-page))
  (let ((href (href page)) (title (link-title page)))
    (<ucw:form :action (add-new-link page :href href :title title)
	       :method "POST"
	       (<:p (<:label :for "title" "Title:")
		    (<ucw:text :accessor title :id "title"))
	       (<:p (<:label :for "href" "URL:")
		    (<ucw:text :accessor href :id "href"))
	       (<:input :type "submit" :value "i Reddit"))))

(defaction add-new-link ((page submit-link-page) &key href title)
  (add-link :href href :title title)
  (answer href))